@@ -15,7 +15,9 @@ class SomeError(Exception):
 
 def ncvartypeoffset(ncfile,var):
     """ purpose: get binary data type and offset of a variable in netcdf file
-        unfortunately, getting these properties are not explicitely implemented in scipy, but most of this code is stolen from scipy: /usr/lib/python2.7/dist-packages/scipy/io/netcdf.py
+        unfortunately, getting these properties are not explicitely implemented in 
+        scipy, but most of this code is adopted from scipy: 
+        /usr/lib/python2.7/dist-packages/scipy/io/netcdf.py
         ncfile is a scipy.io.netcdf.netcdf_file
         var variable we want to calculate the offset from
     """
@@ -172,7 +174,8 @@ def readicecubeps(fstream,shp,refiter,dimiter,dimiterpos,refnoiter,dimnoiter,vty
 
 # we kunnen eens proberen om een variabele aan te maken met een vooraf gespecifieerde dimensie!
 
-def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000, forcearray=False):
+def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000, 
+        forcearray=False, order = [], squeeze=False):
     """ purpose (see also README.md): process binary NetCDF data streams
     func: the function/analysis to be applied on the data input streams
           specified below. The arguments  of func correspond to the respective
@@ -215,12 +218,15 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000, for
                (inner dimensions; dimensions in front). 
 
     maxmembytes: The option is applicable when using appenddim. The maximum amount of bytes that is allowed for the buffers to be read/written from/to the input/output data streams. 
+    order: specify the order of the output dimensions (optional)
+    squeeze: squeeze the output variables: remove the dimensions of length unity for each of the output variables for having cleaner output
     """
 
     # track the dimensions to be selected when multiples are given
     dnamseldef = []
     for ednamsel in dnamsel:
         if ((type(ednamsel).__name__ == 'tuple') | (type(ednamsel).__name__ == 'list')):
+            # I think this type of manipulation is deprecated by the 'daliases'-option
             dnamseldef.append(ednamsel[0])
             for idatin,edatin in reversed(list(enumerate(datin))):
                 ncfn = None
@@ -354,8 +360,11 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000, for
             vsdout[idatout]['itemsize'] = None
             vsdout[idatout]['dtype'] = None
     
-    # collecting the involved dimensions (will be considered as the standard output dimensions)
-    dnamsstd = [] # standard output dimensions: list of all output dimensions: this is collected from the input dimensions, the output dimensions and the selected/processed dimensions
+    # collecting the involved dimensions (will be considered as the standard output 
+    # dimensions)
+    dnamsstd = [] # standard output dimensions: list of all output dimensions: this is 
+                  # collected from the input dimensions, the output dimensions and the 
+                  # selected/processed dimensions
     dimsstd = [] # maximum length of an output dimension
     idimsstd = 0
     
@@ -365,7 +374,10 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000, for
         for idnam,ednam in reversed(list(enumerate(evsdin['dnams']))):
             if ednam not in dnamsstd:
                 # In dnamsstd, ednam should be just after the dimensions preceding ednams in dnams  
-                # # actually, we also want that, in dnamsstd, ednam should be just before the dimensions succeeding ednams in dnams. Sometimes, this is not possible at the same time. But it will be the case if that is possible when applying one of the criteria
+                # actually, we also want that, in dnamsstd, ednam should be just before
+                # the dimensions succeeding ednams in dnams. Sometimes, this is not 
+                # possible at the same time. But it will be the case if that is possible 
+                # when applying one of the criteria
                 idx = 0
                 # implement this also below, I mean that 
                 for idnam2,ednam2 in enumerate(dnamsstd):
@@ -380,7 +392,8 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000, for
                     else:
                         dimsstd.insert(idx  ,int(vsdin[ivsdin]['dims'][idnam]))
                 else:
-                    # In this case, wait for assigning the output dimensions. This actually depends on the specified function
+                    # In this case, wait for assigning the output dimensions. 
+                    # This actually depends on the specified function
                     dimsstd.insert(idx  ,None)
             else:
                 if ((len(vsdin[ivsdin]['dsel'][idnam]) != 1) & (dimsstd[dnamsstd.index(ednam)] != 1) & \
@@ -400,12 +413,9 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000, for
                         else:
                             dimsstd[dnamsstd.index(ednam)] = max(dimsstd[dnamsstd.index(ednam)],vsdin[ivsdin]['dims'][idnam])
     
+
     # add the missing dimensions selected for the function. 
     idnam = 0
-    
-    # TODO: the position of this dimension needs to be accordence to the
-    #       dimension-order suggested by the input(output?) netcdf files!  
-
     for idnamseldef,ednamseldef in enumerate(dnamseldef):
         if ednamseldef not in dnamsstd:
             dnamsstd.insert(idnam,ednamseldef)
@@ -413,14 +423,31 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000, for
             idnam = idnam +1 
         else:
             idnam = dnamsstd.index(ednam)+1
+
+    # stillTODO: the position of this dimension needs to be accordance to the
+    #       dimension-order suggested by the input(output?) netcdf files!  <C-F6>
+
+    # for now, we introduce the option for manually prescribing the order of output dimensions
+    for iorder,eorder in enumerate(order):
+        if eorder in dnamsstd:
+            for iorder2 in range(iorder,len(order)):
+                eorder2 = order[iorder2]
+                if eorder2 in dnamsstd:
+                    if dnamsstd.index(eorder) > dnamsstd.index(eorder2):
+                        dnamsstd.insert(dnamsstd.index(eorder2),dnamsstd.pop(dnamsstd.index(eorder)))
+
     
     #print ("dimsstd", dimsstd)
     
     # dimsstd: list the specific output dimensions
-    # if function dimension: data output dimension should be the same as the function output dimension, but this should be checked afterwards.
+    # if function dimension: data output dimension should be the same as the function 
+    #                        output dimension, but this should be checked afterwards.
     # if not function dimension:
-    # # look what's the output dimension like. If the dimension is not in the output variable, we add a dummy 1-dimension
-    # we need to create/list adimsstd also before!! And then append them with the missing dimensions, as dummy 1-dimensions. If that is not sufficient, we will just get an error message.
+    # # look what's the output dimension like.  
+    # # If the dimension is not in the output variable, we add a dummy 1-dimension
+    # we need to create/list adimsstd also before!! 
+    # And then append them with the missing dimensions, as dummy 1-dimensions. 
+    # If that is not sufficient, we will just get an error message.
     
     # get references to the standard output dimensions on which the function is applied
     refdfuncstd = []
@@ -603,7 +630,8 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000, for
                         datout[iddout]['file'].createDimension(ednams,vsdout[iddout]['dims'][idnams])
                     # if  a template file needed to be opened, close it again
 
-            # check here whether the output dimensions are ok , otherwise make alternative dimensions
+            # check here whether the output dimensions are ok , otherwise 
+            # make alternative dimensions
             dnams = []
             for idim,edim in enumerate(vsdout[iddout]['dims']):
                 if (vsdout[iddout]['dims'][idim] == datout[iddout]['file'].dimensions[vsdout[iddout]['dnams'][idim]]):
@@ -622,7 +650,21 @@ def pcd(func,dnamsel,datin,datout,appenddim = False, maxmembytes = 10000000, for
                     datout[iddout]['file'].createDimension(dnams[idim],vsdout[iddout]['dims'][idim])
             vsdout[iddout]['dnams'] = dnams
 
-            datout[iddout]['file'].createVariable(datout[iddout]['varname'],vsdout[iddout]['dtype'][1],tuple(vsdout[iddout]['dnams']))
+            # squeeze the output variables: remove their dimensions for those having size of unity
+            if squeeze:
+                dnamsout = []
+                for idnam,ednam in enumerate(vsdout[iddout]['dnams']):
+                    if vsdout[iddout]['dims'][idnam] > 1:
+                        dnamsout.append(ednam)
+                # we need at least one dimension
+                if dnamsout == []:
+                    dnamsout.append(vsdout[iddout]['dnams'][0])
+
+            else:
+                dnamsout = vsdout[iddout]['dnams']
+            datout[iddout]['file'].createVariable(datout[iddout]['varname'],\
+                                                  vsdout[iddout]['dtype'][1],\
+                                                  tuple(dnamsout))
             # we should check this at the time the dimensions are not created
             if (vsdout[iddout]['dims'] != list(datout[iddout]['file'].variables[datout[iddout]['varname']].shape)):
                 raise SomeError("dimensions of output file ( "+str(vsdout[iddout]['dims'])+"; "+ str(vsdout[iddout]['dnams'])+") do not correspond with intended output dimension "+str(datout[iddout]['file'].variables[datout[iddout]['varname']].shape)+"; "+str(datout[iddout]['file'].variables[datout[iddout]['varname']].dimensions))
